﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.Editors;
using System.Collections;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            ObservableCollection<MyData> data = new ObservableCollection<MyData>();
            data.Add(new MyData() { Count = 5.0, CountString = "five" });
            data.Add(new MyData() { Count = 1.0, CountString = "one" });
            data.Add(new MyData() { Count = 9.0, CountString = "nine" });

            this.DataContext = data;
        }
    }

    public class MyData
    {
        public double Count { get; set; }
        public string CountString { get; set; }
    }

    public class MyGroupByEvaluator : IGroupByEvaluator
    {
        private MyGroupSortComparer _comparer = new MyGroupSortComparer();

        public bool DoesGroupContainRecord(GroupByRecord groupByRecord, DataRecord record)
        {
            return GetCountString( record ) == groupByRecord.Value as string ;
        }

        public object GetGroupByValue(GroupByRecord groupByRecord, DataRecord record)
        {
            return GetCountString(record);
        }

        public System.Collections.IComparer SortComparer
        {
            get { return _comparer; }
        }

        private static string GetCountString(DataRecord record)
        {
            var item = record.DataItem as MyData;

            if (item != null)
                return item.CountString;

            return record.Cells["Count"].Value.ToString();
        }
    }

    public class MyGroupSortComparer : IComparer
    {
        public int Compare(object x, object y)
        {
            Cell xCell = x as Cell;
            Cell yCell = y as Cell;

            if (x == y)
                return 0;

            if (x == null)
                return -1;

            if (y == null)
                return 1;

            var xValue = xCell.Value as IComparable;
            var yValue = yCell.Value as IComparable;

            return xValue.CompareTo(yValue);
        }
    }

    public class DoubleToStringConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            //DataRecord dr = parameter as DataRecord;
            //if (dr != null && dr.DataItem != null)
            //{
            //    return (dr.DataItem as MyData).CountString;
            //}
            DataRecordPresenter drp = Utilities.GetAncestorFromType(parameter as DependencyObject, typeof(DataRecordPresenter), true) as DataRecordPresenter;
            if ((drp.Record as DataRecord).DataItemIndex >= 0)
            {
                return ((drp.Record as DataRecord).DataItem as MyData).CountString;
            }

            return value.ToString();
            //return Binding.DoNothing;            
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            //TODO: add some logic to convert from a string back to a double

            if ( targetType == typeof(double))
            {
                string text = value as string;

                if (text != null)
                {
                    double dbl;
                    if (double.TryParse(text, out dbl))
                        return dbl;
                }
            }

            return Binding.DoNothing;
        }
    }

}
